@@ -1,7 +1,7 @@
 package Sys::RunAlone;
 
 # version info
-$VERSION = '0.09';
+$VERSION = '0.10';
 
 # make sure we're strict and verbose as possible
 use strict;
@@ -10,8 +10,9 @@ use warnings;
 # make sure we know how to lock
 use Fcntl ':flock';
 
-# silent flag
+# process local storage
 my $silent;
+my $retry;
 
 # satisfy -require-
 1;
@@ -24,9 +25,30 @@ my $silent;
 # import
 #
 #  IN: 1 class (not used)
-#      2 option (default: none)
+#      2 .. N options (default: none)
 
-sub import { $silent= 1 if $_[1] and $_[1] eq 'silent' } #import
+sub import {
+    shift;
+
+    # support obsolete form of silencing
+    $silent = 1, return if @_ == 1 and $_[0] and $_[0] eq 'silent';
+
+    # huh?
+    die "Must specify even number of parameters" if ( @_ & 1 ) == 1;
+
+    # obtain parameters
+    my %args = @_;
+    $silent = delete $args{silent};
+    $retry  = delete $args{retry};
+    $retry  = $ENV{RETRY_SYS_RUNALONE} if exists $ENV{RETRY_SYS_RUNALONE};
+
+    # sanity check
+    if ( my @huh= sort keys %args ) {
+        die "Don't know what to do with: @huh";
+    }
+
+    return;
+} #import
 
 #-------------------------------------------------------------------------------
 
@@ -48,9 +70,25 @@ INIT {
 
     # we're not alone!
     elsif ( !flock main::DATA, LOCK_EX | LOCK_NB ) {
+
+        # need to retry
+        if ($retry) {
+            my ( $times, $sleep )= split ',', $retry;
+            $sleep ||= 1;
+            while ( $times-- ) {
+                sleep $sleep;
+
+                # we're alone!
+                goto ALLOK if flock main::DATA, LOCK_EX | LOCK_NB;
+            }
+        }
+
+        # we're done
         print STDERR "A copy of '$0' is already running\n" if !$silent;
         exit 1;
     }
+
+  ALLOK:
 } #INIT
 
 #-------------------------------------------------------------------------------
@@ -66,17 +104,27 @@ Sys::RunAlone - make sure only one invocation of a script is active at a time
  use Sys::RunAlone;
  # code of which there may only be on instance running on system
 
- use Sys::RunAlone 'silent';
+ use Sys::RunAlone silent => 1;
  # be silent if other running instance detected
 
+ use Sys::RunAlone retry => 50;
+ # retry execution 50 times with wait time of 1 second in between
+
+ use Sys::RunAlone retry => '55,60';
+ # retry execution 55 times with wait time of 60 seconds in between
+
+ use Sys::RunAlone 'silent';
+ # obsolete form of silent => 1
+
 =head1 DESCRIPTION
 
 Provide a simple way to make sure the script from which this module is
-loaded, is only running once on the server.
+loaded, is only running once on the server.  Optionally allow for retrying
+execution until the other instance of the script has finished.
 
 =head1 VERSION
 
-This documentation describes version 0.09.
+This documentation describes version 0.10.
 
 =head1 METHODS
 
@@ -93,11 +141,41 @@ exits with an error message on STDERR and an exit value of 2.
 
 If the DATA handle is available, and it cannot be C<flock>ed, it exits
 with an error message on STDERR and an exit value of 1.  The error message
-will be surpressed when C<'silent'> was specified in the C<use> statement.
+will be surpressed when C<silent => 1> was specified in the C<use> statement.
 
 If there is a DATA handle, and it could be C<flock>ed, execution continues
 without any further interference.
 
+=head1 TRYING MORE THAN ONCE
+
+Optionally, it is possibly to specify a number of retries to be done if the
+first C<flock> fails.  This can be done by either specifying the retry value
+in the C<use> statement as e.g. C<retry => 55>, or with the environment
+variable C<RETRY_SYS_RUNALONE>.  There are two forms of the retry value:
+
+=over 4
+
+=item times
+
+ use Sys::RunAlone retry => 55;  # retry 55 times, with 1 second intervals
+
+Specify the number of times to retry, with 1 second intervals.
+
+=item times,seconds
+
+ use Sys::RunAlone retry => '55,60'; # retry 55 times, with 60 second intervals
+
+Specify both the number of retries as well as the number of seconds interval
+between tries.
+
+=back
+
+This is particularly useful for minutely and hourly scripts that run a long
+and sometimes run into the next period.  Instead of then not doing anything
+for the next period, it will start processing again as soon as it is possible.
+This makes the chance of catching up so that the period after the next period
+everything is in sync again.
+
 =head1 OVERRIDING CHECK
 
 In some cases, the same script may need to be run simultaneously with another
@@ -143,6 +221,9 @@ same time.
 Inspired by Randal Schwartz's mention of using the DATA handle as a semaphore
 on the London PM mailing list.
 
+Booking.com for using this heavily in production and allowing me to improve
+this module.
+
 =head1 SEE ALSO
 
 L<Sys::RunAlways>.
@@ -153,7 +234,7 @@ L<Sys::RunAlways>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005, 2006, 2008, 2009 Elizabeth Mattijsen <liz@dijkmat.nl>.
+Copyright (c) 2005, 2006, 2008, 2009, 2011 Elizabeth Mattijsen <liz@dijkmat.nl>.
 All rights reserved.  This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
 
